from django.conf import settings
from django.http import HttpResponseForbidden, HttpResponseRedirect
from django.shortcuts import render
from django.core.signing import TimestampSigner, BadSignature, SignatureExpired
from django.contrib.auth import get_user_model, login as django_login, REDIRECT_FIELD_NAME
from django.contrib.auth.decorators import login_required
from django.views.decorators.http import require_POST
from django.shortcuts import get_object_or_404
from django.utils.http import is_safe_url
from . import MAX_AGE_OF_SIGNATURE_IN_SECONDS, SESSION_USER_KEY, SESSION_REDIRECT_KEY, can_cloak_as

# no permissions necessary since this only works for valid signatures
def login(request, signature):
    """
    Automatically logs in a user based on a signed PK of a user object. The
    signature should be generated with the `login` management command.

    The signature will only work for 60 seconds.
    """
    signer = TimestampSigner()
    try:
        pk = signer.unsign(signature, max_age=MAX_AGE_OF_SIGNATURE_IN_SECONDS)
    except signing.BadSignature, SignatureExpired:
        return HttpResponseForbidden("Can't log you in")

    user = get_object_or_404(get_user_model(), pk=pk)
    # we *have* to set the backend for this user, so we just use the first one
    user.backend = settings.AUTHENTICATION_BACKENDS[0]
    django_login(request, user)

    return HttpResponseRedirect(settings.LOGIN_REDIRECT_URL)

@login_required
@require_POST
def cloak(request, pk=None):
    """
    Masquerade as a particular user and redirect based on the
    REDIRECT_FIELD_NAME parameter, or the LOGIN_REDIRECT_URL.

    Callers can either pass the pk of the user in the URL itself, or as a POST
    param.
    """
    pk = request.POST.get('pk', pk)
    if pk is None:
        return HttpResponse("You need to pass a pk POST parameter, or include it in the URL")

    user = get_object_or_404(get_user_model(), pk=pk)

    if not can_cloak_as(request.user, user):
        return HttpResponseForbidden("You are not allowed to cloak as this user")

    request.session[SESSION_USER_KEY] = user.pk
    # save the referer information so when uncloaking, we can redirect the user
    # back to where they were
    request.session[SESSION_REDIRECT_KEY] = request.META.get("HTTP_REFERER", settings.LOGIN_REDIRECT_URL)

    # redirect the cloaked user to the URL specified in the "next" parameter,
    # or to the default redirect URL
    return HttpResponseRedirect(request.POST.get(REDIRECT_FIELD_NAME, settings.LOGIN_REDIRECT_URL))

# no perms neccessary here
@require_POST
def uncloak(request):
    """
    Undo a masquerade session and redirect the user back to where they started
    cloaking from (or where ever the "next" POST parameter points)
    """
    try:
        del request.session[SESSION_USER_KEY]
    except KeyError:
        pass # who cares

    # figure out where to redirect
    next = request.POST.get(REDIRECT_FIELD_NAME) or request.session.get(SESSION_REDIRECT_KEY)
    if next and is_safe_url(next, request.get_host()):
        return HttpResponseRedirect(next)
    return HttpResponseRedirect(settings.LOGIN_REDIRECT_URL)



from __future__ import print_function
from django.core.management.base import BaseCommand, CommandError
from django.contrib.auth import get_user_model 
from django.core.signing import TimestampSigner
from django.core.urlresolvers import reverse
from django.core.exceptions import FieldError
from ...views import login

class Command(BaseCommand):
    args = '[USERNAME_FIELD value]'
    help = 'Login as a user using a temporary URL'

    def handle(self, *args, **options):
        """
        With no arguments, find the first user in the system with the is_admin
        flag set to true, or just the first user in the system period.

        With a single argument, look for the user with that value as the
        USERNAME_FIELD value.

        When a user is found, print out a URL slug you can paste into your
        browser to login as the user.
        """
        user_model = get_user_model()

        if len(args) == 0:
            # find the first admin user, or the first user ordered by PK
            try:
                user_model.objects.filter(is_admin=True).order_by("pk").first()
            except FieldError as e:
                # now just try to get first user ordered by pk 
                user = user_model.objects.all().order_by("pk").first()

            if user is None:
                raise CommandError("No users found!")

        elif len(args) == 1:
            # find the user with the USERNAME_FIELD equal to the command line
            # argument
            try:
                user = user_model.objects.get_by_natural_key(args[0])
            except user_model.DoesNotExist as e:
                raise CommandError("The user does not exist")
        else:
            raise CommandError("You passed me too many arguments")

        signer = TimestampSigner()
        signature = signer.sign(str(user.pk))

        print(reverse(login, args=(signature,)))



from django.contrib.auth import get_user_model
from . import SESSION_USER_KEY, can_cloak_as

class CloakMiddleware(object):
    """
    This middleware class checks to see if a cloak session variable is
    set, and overrides the request.user object with the cloaked user
    """
    def process_request(self, request):
        request.user.is_cloaked = False
        if SESSION_USER_KEY in request.session:
            try:
                user = get_user_model().objects.get(pk=request.session[SESSION_USER_KEY])
            except User.DoesNotExist:
                return None

            if can_cloak_as(request.user, user):
                request.user = user
                request.user.is_cloaked = True



#!/usr/bin/env python
from shutil import rmtree
from setuptools import setup

setup(
    name="django-cloak",
    version=__import__('cloak').__version__,
    url='https://github.com/PSU-OIT-ARC/django-cloak',
    author='Matt Johnson',
    author_email='mdj2@pdx.edu',
    description="App for Django to cloak as a user, or generate a login link",
    packages=['cloak','cloak.management', 'cloak.management.commands'],
    zip_safe=False,
    install_requires=[
        'django',
    ],
    classifiers=[
        'Framework :: Django',
    ],
)



from django.conf.urls import patterns, url
from .views import cloak, uncloak, login

urlpatterns = patterns('',
    url(r'^cloak$', cloak, name="cloak"),
    url(r'^cloak/(?P<pk>.+)$', cloak, name="cloak"),
    url(r'^uncloak$', uncloak, name="uncloak"),
    url(r'^login/(?P<signature>.*)$', login),
)



__version__ = "0.1.2"

MAX_AGE_OF_SIGNATURE_IN_SECONDS = 60
SESSION_USER_KEY = "_cloak"
SESSION_REDIRECT_KEY = "_cloak_redirect"

def can_cloak_as(user, other_user):
    """
    Returns true if `user` can cloak as `other_user`
    """
    # check to see if the user is allowed to do this
    can_cloak = False
    try:
        can_cloak = user.can_cloak_as(other_user)
    except AttributeError as e:
        try:
            can_cloak = user.is_admin
        except AttributeError as e:
            pass

    return can_cloak
